/*======================================================*/
/*														*/
/*	utauVoiceDataBase.cpp								*/
/*						(c) HAL 2009-					*/
/*														*/
/*======================================================*/
#include "utauVoiceDataBase.h"

static stBool callbackFunc(Straight straight, stCallbackType callbackType,
                           void *callbackData, void *userData)
{
    int percent;
    
    if (callbackType == STRAIGHT_F0_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT F0: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_AP_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT AP: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_SPECGRAM_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT spectrogram: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_SYNTH_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT synthesis: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    }
	return ST_TRUE;
}

utauVoiceDataBase::~utauVoiceDataBase()
{
	list<utauParameters*>::iterator i;
	for( i = settingList.begin(); i != settingList.end(); i++ )
		if( (*i) )
			delete (*i);
}

VCNT_RESULT	utauVoiceDataBase::readUtauVoiceDataBase( string fileName )
{
	VCNT_RESULT result = VCNT_FILE_NOT_FOUND;
	FILE* fp;
	char buf[LINEBUFF_LEN];
	string temp;
	int index;

#ifdef __GNUC__
	fp = fopen( fileName.c_str(), "r" );
#else
	fopen_s( &fp, fileName.c_str(), "r" );
#endif

	if( fp ){
		voicePath = fileName.substr( 0, fileName.rfind( PATH_SEPARATOR ) + 1 );

		while( c_getline( buf, LINEBUFF_LEN, fp ) != NULL ){
			temp = buf;
			utauParameters* current = new utauParameters;
			if( current ){
				if( ( index = temp.find( ".wav" ) ) == string::npos )
					if( ( index = temp.find( ".stf" ) ) == string::npos )
						index = temp.find( "." );
				current->fileName = temp.substr( 0, index );
				/* When no lyric symbol exists, voiceDB will use fileName instead. */
				if( temp.find( "=" ) + 1 == temp.find( "," ) )
					current->lyric = current->fileName;
				else{
					current->lyric = temp.substr( temp.find( "=" ) + 1 );
					current->lyric = current->lyric.substr( 0, current->lyric.find( "," ) );
				}

				temp = temp.substr( temp.find( "," ) + 1 );
				current->msLeftBlank = (float)atof( temp.c_str() );

				temp = temp.substr( temp.find( "," ) + 1 );
				current->msFixedLength = (float)atof( temp.c_str() );

				temp = temp.substr( temp.find( "," ) + 1 );
				current->msRightBlank = (float)atof( temp.c_str() );

				temp = temp.substr( temp.find( "," ) + 1 );
				current->msPreUtterance = (float)atof( temp.c_str() );

				temp = temp.substr( temp.find( "," ) + 1 );
				current->msVoiceOverlap = (float)atof( temp.c_str() );

				settingMap.insert( make_pair( current->lyric, current ) );
				if( current->lyric.compare( current->fileName ) != 0 )
					settingMap.insert( make_pair( current->fileName, current ) );
				settingList.push_back( current );

			}else{
				result = VCNT_OUT_OF_MEMORY;
				break;
			}
		}
		if( result != VCNT_OUT_OF_MEMORY )
			result = VCNT_TRUE;
		fclose( fp );
	}
	return result;
}

VCNT_RESULT	utauVoiceDataBase::getUtauParameters( utauParameters &parameters, string search )
{
	VCNT_RESULT	result = VCNT_FALSE;
	MAP_TYPE<string,utauParameters*>::iterator i = settingMap.find( search );
	if( i != settingMap.end() ){
		if( i->second ){
			parameters.fileName = i->second->fileName;
			parameters.lyric = i->second->lyric;
			parameters.msFixedLength = i->second->msFixedLength;
			parameters.msLeftBlank = i->second->msLeftBlank;
			parameters.msPreUtterance = i->second->msPreUtterance;
			parameters.msRightBlank = i->second->msRightBlank;
			parameters.msVoiceOverlap = i->second->msVoiceOverlap;
			result = VCNT_TRUE;
		}
	}
	return result;
}

VCNT_RESULT utauVoiceDataBase::straightAnalysis( void )
{
	VCNT_RESULT		result = VCNT_FALSE;
	bool			init_here = false;
	straightFileEx	strFile;
	waveFileEx		waveFile;
	vector<double>	buffer;
	char			buf[9];
	string			s;
	int				count = 0;

	FILE*			fp;

	if( !straight ){
#ifdef __STRAIGHT_TANDEM__
		straightInitConfigTandem( &config );
#else
		straightInitConfig( &config );
#endif
		config.samplingFrequency = 44100.0;
		straight = straightInitialize( &config );
		init_here = true;
		if( straight )
			straightSetCallbackFunc(straight, STRAIGHT_PERCENTAGE_CALLBACK, callbackFunc, NULL);
	}

	string analyzed_dir	= voicePath + "analyzed";
	string analyzed_oto = analyzed_dir + PATH_SEPARATOR + "oto.ini";
	c_mkdir( analyzed_dir.c_str() );

#ifdef __GNUC__
	fp = fopen( analyzed_oto.c_str(), "w" );
#else
	fopen_s( &fp, analyzed_oto.c_str(), "w" );
#endif

	if( straight && fp ){
		waveFileEx	waveTemp;
		for( list<utauParameters*>::iterator i = settingList.begin(); i != settingList.end(); i++ ){
			if( (*i) ){
				cout << "begin analysis: " << (*i)->lyric << endl;
#ifdef __GNUC__
				sprintf( buf, "%d", count );
#else
				sprintf_s( buf, 9, "%d", count );
#endif
				s = buf;
				waveFile.readWaveFile( voicePath + (*i)->fileName + ".wav" );
				waveFile.getWaveBuffer( buffer, (*i)->msLeftBlank - config.frameLength / 2.0 , (*i)->msRightBlank - config.frameLength / 2.0 );
				waveTemp.setWaveBuffer( buffer );
				strFile.straightCompute( waveTemp );
				strFile.writeStfFile( voicePath + "analyzed" + PATH_SEPARATOR + s + ".stf" );

				fprintf( fp, "%d.stf=%s,%6f,%6f,%6f,%6f,%6f\n", count, (*i)->lyric.c_str(), config.frameLength / 2.0, (*i)->msFixedLength, 0.0, (*i)->msPreUtterance, (*i)->msVoiceOverlap );

				cout << "analysis done." << endl;
				cout << "wrote the file: " << s << ".stf" << endl;

				count++;
			}
		}
		result = VCNT_TRUE;
	}

	if( fp )
		fclose( fp );

	if( init_here && straight ){
		straightDestroy( straight );
		straight = NULL;
	}

	return result;
}

