#include "vConnectFileManager.h"

vConnectFileManager::~vConnectFileManager()
{
	/* Release all files. */
	for( MAP_TYPE<string,straightFileInfo*>::iterator i = fileInfo.begin(); i != fileInfo.end(); i++ )
		if( i->second )
			delete i->second;
}

VCNT_RESULT	vConnectFileManager::setPronounces( long frameIndex, vector<vConnectPronounce>& pronounces )
{
	VCNT_RESULT result = VCNT_TRUE;
	MAP_TYPE<string,straightFileInfo*>::iterator i;

	for( unsigned int n = 0; n < pronounces.size(); n++ ){
		i = fileInfo.find( pronounces[n].stfFileName );
		if( i != fileInfo.end() ){
			if( i->second )
				i->second->endFrame = frameIndex;
		}else{
			straightFileInfo* temp = new straightFileInfo;
			if( temp ){
				temp->endFrame = frameIndex;
				temp->isInitialized = false;
				fileInfo.insert( make_pair( pronounces[n].stfFileName, temp ) );
			}
		}
	}
	return result;
}

VCNT_RESULT vConnectFileManager::getFrame( unsigned long frameIndex, vConnectPronounce& pronounce, straightFrameEx& frame )
{
	VCNT_RESULT	result = VCNT_FALSE;
	MAP_TYPE<string,straightFileInfo*>::iterator i = fileInfo.find( pronounce.stfFileName );
	unsigned long frameNumber,length;

	if( i != fileInfo.end() )
		if( i->second ){
			if( !(i->second->isInitialized) )
				i->second->isInitialized = ( VCNT_TRUE == i->second->file.readStfFile( pronounce.stfFileName ) );

			if( i->second->isInitialized ){
				length = i->second->file.getNumFrames();
				frameNumber = pronounce.absolute + (long)( (double)( length - pronounce.absolute - pronounce.rightBlank ) * pronounce.relative );
				result = i->second->file.getFrame( frameNumber, frame );
				/* When frameIndex is endFrame, release fileInfo and erase from map. */
				/* This class expects frameIndex comes in order. */
				if( i->second->endFrame == frameIndex ){
					delete i->second;
					fileInfo.erase( i );
				}
			}
		}

	return result;
}

VCNT_RESULT	vConnectFileManager::getFrame( unsigned long frameIndex, vector<vConnectPronounce>& pronounces, vector<straightFrameEx>& frames )
{
	VCNT_RESULT	result = VCNT_TRUE;

	unsigned int count = 0;
	for( count = 0; count < pronounces.size(); count++ )
		if( pronounces[count].morphRate != 0.0 )
			break;

	if( count == pronounces.size() ){
		frames.clear();
	} else {

		frames.resize( pronounces.size() );
		for( unsigned int i = 0; i < pronounces.size(); i++ ){
			/* Calculate frameIndex here. */
			if( pronounces[i].stfFileName.compare( "" ) != 0 || pronounces[i].morphRate == 0.0 ){
				getFrame( frameIndex, pronounces[i], frames[i] );
				frames[i].multipleBuffer( pronounces[i].morphRate );
			} else {
				pronounces[i].morphRate = 0.0;
				frames[i].multipleBuffer( 0.0 );
			}
		}
	}

	return result;
}

